@@ -9,13 +9,13 @@ use Log::ger;
 use List::Util qw(shuffle);
 
 our $AUTHORITY = 'cpan:PERLANCAR'; # AUTHORITY
-our $DATE = '2021-10-24'; # DATE
+our $DATE = '2022-07-22'; # DATE
 our $DIST = 'App-wordlist'; # DIST
-our $VERSION = '0.281'; # VERSION
+our $VERSION = '0.282'; # VERSION
 
 our %SPEC;
 
-our %arg_wordlists = (
+our %argspecopt_wordlists = (
     wordlists => {
         'x.name.is_plural' => 1,
         schema => ['array*' => {
@@ -34,6 +34,31 @@ our %arg_wordlists = (
                 ci    => 1,
             );
         },
+        tags => ['category:module-selection'],
+    },
+);
+
+our %argspecopt_wordlist_bundles = (
+    wordlist_bundles => {
+        'x.name.is_plural' => 1,
+        'x.name.singular' => 'wordlist_bundle',
+        schema => ['array*' => {
+            of => 'str*', # for the moment we need to use 'str' instead of 'perl::modname' due to Perinci::Sub::GetArgs::Argv limitation
+            'x.perl.coerce_rules'=>[ ['From_str_or_array::expand_perl_modname_wildcard'=>{ns_prefix=>"WordList"}] ],
+        }],
+        summary => 'Select one or more wordlist bundle (Acme::CPANModules::WordListBundle::*) modules',
+        cmdline_aliases => {b=>{}},
+        element_completion => sub {
+            require Complete::Util;
+
+            my %args = @_;
+            Complete::Util::complete_array_elem(
+                word  => $args{word},
+                array => [map {$_->{name}} @{ _list_installed_bundles() }],
+                ci    => 1,
+            );
+        },
+        tags => ['category:module-selection'],
     },
 );
 
@@ -70,6 +95,34 @@ sub _list_installed {
     \@res;
 }
 
+sub _list_installed_bundles {
+    require Module::List;
+    my $mods = Module::List::list_modules(
+        "Acme::CPANModules::WordListBundle::",
+        {
+            list_modules  => 1,
+            list_pod      => 0,
+            recurse       => 1,
+            return_path   => 1,
+        });
+    my @res;
+    for my $wlb0 (sort keys %$mods) {
+        (my $wlb = $wlb0) =~ s/\AAcme::CPANModules::WordListBundle:://;
+
+        my $lang = '';
+        if ($wlb =~ /^(\w\w)::/) {
+            $lang = $1;
+        }
+
+        push @res, {
+            name => $wlb,
+            lang => $lang,
+            path => $mods->{$wlb0}{module_path},
+        };
+     }
+    \@res;
+}
+
 sub _word_has_chars_unordered {
     my ($word, $chars, $ci) = @_;
 
@@ -114,20 +167,25 @@ $SPEC{wordlist} = {
             schema => ['array*' => of => 'str*'],
             pos => 0,
             greedy => 1,
+            tags => ['category:word-filtering'],
         },
         ignore_case => {
             schema  => 'bool',
             default => 1,
             cmdline_aliases => {i=>{}},
+            tags => ['category:word-filtering'],
         },
         len => {
             schema  => 'int*',
+            tags => ['category:word-filtering'],
         },
         min_len => {
             schema  => 'int*',
+            tags => ['category:word-filtering'],
         },
         max_len => {
             schema  => 'int*',
+            tags => ['category:word-filtering'],
         },
         num => {
             summary => 'Return (at most) this number of words (0 = unlimited)',
@@ -147,15 +205,20 @@ _
             cmdline_aliases => {r=>{}},
         },
 
-        %arg_wordlists,
+        %argspecopt_wordlists,
+        %argspecopt_wordlist_bundles,
         or => {
             summary => 'Match any word in query instead of the default "all"',
             schema  => 'bool',
+            tags => ['category:word-filtering'],
         },
         action => {
             schema  => ['str*', in=>[
-                'list_cpan', 'list_installed',
-                'grep', 'stat',
+                'list_cpan',
+                'list_installed',
+                'list_selected',
+                'grep',
+                'stat',
             ]],
             default => 'grep',
             cmdline_aliases => {
@@ -175,6 +238,26 @@ _
                     code => sub { my $args=shift; $args->{action} = 'stat' },
                 },
             },
+            description => <<'_',
+
+Action `list_installed` (shortcut option `-l`) will list WordList::* modules
+installed on the local system.
+
+Action `list_cpan` (shortcut option `-L`) will list available WordList::*
+modules on CPAN, either by querying the MetaCPAN site or by querying a local
+mini CPAN using <pm:App::lcpan>.
+
+Action `list_selected` (option `--action=list_selected`) will list the selected
+WordList::* modules (e.g. via `-w` or `-b`).
+
+Action `grep` (the default action) will filter the words from each selected
+wordlists and print them.
+
+Action `stat` (shortcut option `-s`) will show statistics about all the selected
+wordlists.
+
+
+_
         },
         lcpan => {
             schema => 'bool',
@@ -195,10 +278,12 @@ _
         chars_unordered => {
             summary => 'Specify possible characters for the word (unordered)',
             schema => 'str*',
+            tags => ['category:word-filtering'],
         },
         chars_ordered => {
             summary => 'Specify possible characters for the word (ordered)',
             schema => 'str*',
+            tags => ['category:word-filtering'],
         },
         langs => {
             'x.name.is_plural' => 1,
@@ -224,6 +309,7 @@ _
                 Complete::Util::complete_array_elem(
                     word => $args{word}, array => \@langs);
             },
+            tags => ['category:module-selection'],
         },
         color => {
             summary => 'When to highlight search string/matching pattern with color',
@@ -268,6 +354,24 @@ _
             test => 0,
             'x.doc.show_result' => 0,
         },
+        {
+            argv => [qw/-b Proverbs foo/],
+            summary => 'Select a bunch of wordlists via wordlist bundle (Acme::CPANModules::WordListBundle::* module)',
+            test => 0,
+            'x.doc.show_result' => 0,
+        },
+        {
+            argv => [qw/-w Phrase::**::Proverb::** foo/],
+            summary => 'An alternative to select all proverb wordlists',
+            test => 0,
+            'x.doc.show_result' => 0,
+        },
+        {
+            argv => [qw/-w Phrase::**::Proverb::** --action=list_selected/],
+            summary => 'Check to see which wordlists we are selecting via `-w` with wildcards or via `-b`',
+            test => 0,
+            'x.doc.show_result' => 0,
+        },
         {
             argv => [qw/-w EN::Enable --len 6 -i --chars-unordered bobleg/],
             summary => 'Print all words from EN::Enable wordlist that are 6 characters long and have the letters BOBLEG (in no particular order); great for cheats in word forming games',
@@ -293,12 +397,6 @@ _
             test => 0,
             'x.doc.show_result' => 0,
         },
-        {
-            argv => ['-w', 'MetaSyntactic=theme,dangdut'],
-            summary => 'Select a wordlist with parameters',
-            test => 0,
-            'x.doc.show_result' => 0,
-        },
         {
             argv => [qw/-l/],
             summary => 'List installed wordlist modules',
@@ -333,7 +431,7 @@ sub wordlist {
     my $use_color = ($color eq 'always' ? 1 : $color eq 'never' ? 0 : undef)
         // $ENV{COLOR} // (-t STDOUT);
 
-    if ($action eq 'grep' || $action eq 'stat') {
+    if ($action eq 'grep' || $action eq 'stat' || $action eq 'list_selected') {
         # convert /.../ in arg to regex
         for (@$arg) {
             $_ = Encode::decode('UTF-8', $_);
@@ -345,10 +443,41 @@ sub wordlist {
         }
 
         my @res;
-        my $wordlists;
+        my $wordlists = [];
+        my $has_specified_list;
+
+        if ($args{wordlist_bundles}) {
+            $has_specified_list++;
+            for my $wb (@{ $args{wordlist_bundles} }) {
+                my $wbmod = "Acme::CPANModules::WordListBundle::$wb";
+                (my $wbmodpm = "$wbmod.pm") =~ s!::!/!g;
+                require $wbmodpm;
+
+                my $list;
+                {
+                    no strict 'refs'; ## no critic: TestingAndDebugging::ProhibitNoStrict
+                    $list = ${"$wbmod\::LIST"};
+                }
+
+                my $i = -1;
+                for my $entry (@{ $list->{entries} }) {
+                    $i++;
+                    my $mod = $entry->{module};
+                    $mod =~ s/\AWordList::// or do {
+                        warn "Wordlist bundle module $wbmod: entry[$i]: module is not WordList::, skipped";
+                        next;
+                    };
+                    push @$wordlists, $mod;
+                }
+            } # for $wb
+        }
+
         if ($args{wordlists}) {
-            $wordlists = $args{wordlists};
-        } else {
+            $has_specified_list++;
+            push @{ $wordlists }, @{ $args{wordlists} };
+        }
+
+        unless ($has_specified_list) {
             $wordlists = [];
             for my $rec (@$list_installed) {
                 if ($args{langs} && @{ $args{langs} }) {
@@ -361,6 +490,10 @@ sub wordlist {
         $wordlists = [shuffle @$wordlists] if $random;
         log_trace "Wordlist(s) to use: %s", $wordlists;
 
+        if ($action eq 'list_selected') {
+            return [200, "OK", $wordlists];
+        }
+
         if ($action eq 'stat') {
             no strict 'refs';
             return [200] unless @$wordlists;
@@ -568,7 +701,7 @@ App::wordlist - Grep words from WordList::*
 
 =head1 VERSION
 
-This document describes version 0.281 of App::wordlist (from Perl distribution App-wordlist), released on 2021-10-24.
+This document describes version 0.282 of App::wordlist (from Perl distribution App-wordlist), released on 2022-07-22.
 
 =head1 SYNOPSIS
 
@@ -639,10 +772,6 @@ Examples:
 
  wordlist(arg => ["/fof[aeiou]/"]);
 
-=item * Select a wordlist with parameters:
-
- wordlist(wordlists => ["MetaSyntactic=theme,dangdut"]);
-
 =item * List installed wordlist modules:
 
  wordlist(action => "list_installed");
@@ -661,6 +790,22 @@ Arguments ('*' denotes required arguments):
 
 =item * B<action> => I<str> (default: "grep")
 
+Action C<list_installed> (shortcut option C<-l>) will list WordList::* modules
+installed on the local system.
+
+Action C<list_cpan> (shortcut option C<-L>) will list available WordList::*
+modules on CPAN, either by querying the MetaCPAN site or by querying a local
+mini CPAN using L<App::lcpan>.
+
+Action C<list_selected> (option C<--action=list_selected>) will list the selected
+WordList::* modules (e.g. via C<-w> or C<-b>).
+
+Action C<grep> (the default action) will filter the words from each selected
+wordlists and print them.
+
+Action C<stat> (shortcut option C<-s>) will show statistics about all the selected
+wordlists.
+
 =item * B<arg> => I<array[str]>
 
 =item * B<chars_ordered> => I<str>
@@ -720,6 +865,10 @@ Pick random words.
 If set to true, then streaming will be turned off. All words will be gathered
 first, then words will be chosen randomly from the gathered list.
 
+=item * B<wordlist_bundles> => I<array[str]>
+
+Select one or more wordlist bundle (Acme::CPANModules::WordListBundle::*) modules.
+
 =item * B<wordlists> => I<array[str]>
 
 Select one or more wordlist modules.
@@ -796,7 +945,7 @@ beyond that are considered a bug and can be reported to me.
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2021, 2020, 2018, 2017, 2016, 2015, 2014 by perlancar <perlancar@cpan.org>.
+This software is copyright (c) 2022, 2021, 2020, 2018, 2017, 2016, 2015, 2014 by perlancar <perlancar@cpan.org>.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
